@@ -42,6 +42,7 @@
 #include "gdm-display-glue.h"
 #include "gdm-display-access-file.h"
 #include "gdm-launch-environment.h"
+#include "gdm-remote-display.h"
 
 #include "gdm-settings-direct.h"
 #include "gdm-settings-keys.h"
@@ -93,6 +94,8 @@ typedef struct _GdmDisplayPrivate
         guint                 have_existing_user_accounts : 1;
         guint                 doing_initial_setup : 1;
         guint                 session_registered : 1;
+
+        GStrv                 supported_session_types;
 } GdmDisplayPrivate;
 
 enum {
@@ -116,6 +119,7 @@ enum {
         PROP_HAVE_EXISTING_USER_ACCOUNTS,
         PROP_DOING_INITIAL_SETUP,
         PROP_SESSION_REGISTERED,
+        PROP_SUPPORTED_SESSION_TYPES,
 };
 
 static void     gdm_display_class_init  (GdmDisplayClass *klass);
@@ -165,6 +169,8 @@ gdm_display_get_session_id (GdmDisplay *self)
 {
         GdmDisplayPrivate *priv;
 
+        g_return_val_if_fail (GDM_IS_DISPLAY (self), NULL);
+
         priv = gdm_display_get_instance_private (self);
         return priv->session_id;
 }
@@ -175,13 +181,9 @@ _create_access_file_for_user (GdmDisplay  *self,
                               GError     **error)
 {
         GdmDisplayAccessFile *access_file;
-        GError *file_error;
 
         access_file = gdm_display_access_file_new (username);
-
-        file_error = NULL;
-        if (!gdm_display_access_file_open (access_file, &file_error)) {
-                g_propagate_error (error, file_error);
+        if (!gdm_display_access_file_open (access_file, error)) {
                 return NULL;
         }
 
@@ -192,8 +194,8 @@ gboolean
 gdm_display_create_authority (GdmDisplay *self)
 {
         GdmDisplayPrivate    *priv;
-        GdmDisplayAccessFile *access_file;
-        GError               *error;
+        g_autoptr(GdmDisplayAccessFile) access_file = NULL;
+        g_autoptr(GError) error = NULL;
         gboolean              res;
 
         g_return_val_if_fail (GDM_IS_DISPLAY (self), FALSE);
@@ -201,12 +203,10 @@ gdm_display_create_authority (GdmDisplay *self)
         priv = gdm_display_get_instance_private (self);
         g_return_val_if_fail (priv->access_file == NULL, FALSE);
 
-        error = NULL;
         access_file = _create_access_file_for_user (self, GDM_USERNAME, &error);
 
         if (access_file == NULL) {
                 g_critical ("could not create display access file: %s", error->message);
-                g_error_free (error);
                 return FALSE;
         }
 
@@ -219,15 +219,12 @@ gdm_display_create_authority (GdmDisplay *self)
                                                    &error);
 
         if (! res) {
-
                 g_critical ("could not add display to access file: %s", error->message);
-                g_error_free (error);
                 gdm_display_access_file_close (access_file);
-                g_object_unref (access_file);
                 return FALSE;
         }
 
-        priv->access_file = access_file;
+        priv->access_file = g_steal_pointer (&access_file);
 
         return TRUE;
 }
@@ -253,8 +250,8 @@ gdm_display_add_user_authorization (GdmDisplay *self,
                                     GError    **error)
 {
         GdmDisplayPrivate    *priv;
-        GdmDisplayAccessFile *access_file;
-        GError               *access_file_error;
+        g_autoptr(GdmDisplayAccessFile) access_file = NULL;
+        g_autoptr(GError) access_file_error = NULL;
         gboolean              res;
 
         int                       i;
@@ -262,6 +259,8 @@ gdm_display_add_user_authorization (GdmDisplay *self,
         xcb_void_cookie_t         cookies[3];
 
         g_return_val_if_fail (GDM_IS_DISPLAY (self), FALSE);
+        g_return_val_if_fail (username != NULL, FALSE);
+        g_return_val_if_fail (filename != NULL, FALSE);
 
         priv = gdm_display_get_instance_private (self);
 
@@ -277,13 +276,12 @@ gdm_display_add_user_authorization (GdmDisplay *self,
 
         g_debug ("GdmDisplay: Adding user authorization for %s", username);
 
-        access_file_error = NULL;
         access_file = _create_access_file_for_user (self,
                                                     username,
                                                     &access_file_error);
 
         if (access_file == NULL) {
-                g_propagate_error (error, access_file_error);
+                g_propagate_error (error, g_steal_pointer (&access_file_error));
                 return FALSE;
         }
 
@@ -296,14 +294,13 @@ gdm_display_add_user_authorization (GdmDisplay *self,
                 g_debug ("GdmDisplay: Unable to add user authorization for %s: %s",
                          username,
                          access_file_error->message);
-                g_propagate_error (error, access_file_error);
+                g_propagate_error (error, g_steal_pointer (&access_file_error));
                 gdm_display_access_file_close (access_file);
-                g_object_unref (access_file);
                 return FALSE;
         }
 
         *filename = gdm_display_access_file_get_path (access_file);
-        priv->user_access_file = access_file;
+        priv->user_access_file = g_steal_pointer (&access_file);
 
         g_debug ("GdmDisplay: Added user authorization for %s: %s", username, *filename);
         /* Remove access for the programs run by greeter now that the
@@ -341,6 +338,7 @@ gdm_display_remove_user_authorization (GdmDisplay *self,
         GdmDisplayPrivate *priv;
 
         g_return_val_if_fail (GDM_IS_DISPLAY (self), FALSE);
+        g_return_val_if_fail (username != NULL, FALSE);
 
         priv = gdm_display_get_instance_private (self);
 
@@ -510,13 +508,13 @@ gdm_display_real_prepare (GdmDisplay *self)
         return TRUE;
 }
 
-static void
+static gboolean
 look_for_existing_users_sync (GdmDisplay *self)
 {
         GdmDisplayPrivate *priv;
-        GError *error = NULL;
-        GVariant *call_result;
-        GVariant *user_list;
+        g_autoptr(GError) error = NULL;
+        g_autoptr(GVariant) call_result = NULL;
+        g_autoptr(GVariant) user_list = NULL;
 
         priv = gdm_display_get_instance_private (self);
         priv->accountsservice_proxy = g_dbus_proxy_new_sync (priv->connection,
@@ -528,8 +526,8 @@ look_for_existing_users_sync (GdmDisplay *self)
                                                              &error);
 
         if (!priv->accountsservice_proxy) {
-                g_warning ("Failed to contact accountsservice: %s", error->message);
-                goto out;
+                g_critical ("Failed to contact accountsservice: %s", error->message);
+                return FALSE;
         }
 
         call_result = g_dbus_proxy_call_sync (priv->accountsservice_proxy,
@@ -541,16 +539,14 @@ look_for_existing_users_sync (GdmDisplay *self)
                                               &error);
 
         if (!call_result) {
-                g_warning ("Failed to list cached users: %s", error->message);
-                goto out;
+                g_critical ("Failed to list cached users: %s", error->message);
+                return FALSE;
         }
 
         g_variant_get (call_result, "(@ao)", &user_list);
         priv->have_existing_user_accounts = g_variant_n_children (user_list) > 0;
-        g_variant_unref (user_list);
-        g_variant_unref (call_result);
-out:
-        g_clear_error (&error);
+
+        return TRUE;
 }
 
 gboolean
@@ -568,7 +564,9 @@ gdm_display_prepare (GdmDisplay *self)
         /* FIXME: we should probably do this in a more global place,
          * asynchronously
          */
-        look_for_existing_users_sync (self);
+        if (!look_for_existing_users_sync (self)) {
+                exit (EXIT_FAILURE);
+        }
 
         priv->doing_initial_setup = wants_initial_setup (self);
 
@@ -616,10 +614,8 @@ gdm_display_finish (GdmDisplay *self)
         g_return_val_if_fail (GDM_IS_DISPLAY (self), FALSE);
 
         priv = gdm_display_get_instance_private (self);
-        if (priv->finish_idle_id != 0) {
-                g_source_remove (priv->finish_idle_id);
-                priv->finish_idle_id = 0;
-        }
+
+        g_clear_handle_id (&priv->finish_idle_id, g_source_remove);
 
         _gdm_display_set_status (self, GDM_DISPLAY_FINISHED);
 
@@ -675,8 +671,6 @@ gdm_display_unmanage (GdmDisplay *self)
 
         priv = gdm_display_get_instance_private (self);
 
-        g_debug ("GdmDisplay: unmanage display");
-
         gdm_display_disconnect (self);
 
         if (priv->user_access_file != NULL) {
@@ -912,6 +906,23 @@ _gdm_display_set_allow_timed_login (GdmDisplay     *self,
         priv->allow_timed_login = allow_timed_login;
 }
 
+static void
+_gdm_display_set_supported_session_types (GdmDisplay         *self,
+                                          const char * const *supported_session_types)
+
+{
+        GdmDisplayPrivate *priv;
+        g_autofree char *supported_session_types_string = NULL;
+
+	if (supported_session_types != NULL)
+          supported_session_types_string = g_strjoinv (":", (GStrv) supported_session_types);
+
+        priv = gdm_display_get_instance_private (self);
+        g_debug ("GdmDisplay: supported session types: %s", supported_session_types_string);
+        g_strfreev (priv->supported_session_types);
+        priv->supported_session_types = g_strdupv ((GStrv) supported_session_types);
+}
+
 static void
 gdm_display_set_property (GObject        *object,
                           guint           prop_id,
@@ -968,6 +979,9 @@ gdm_display_set_property (GObject        *object,
         case PROP_SESSION_REGISTERED:
                 _gdm_display_set_session_registered (self, g_value_get_boolean (value));
                 break;
+        case PROP_SUPPORTED_SESSION_TYPES:
+                _gdm_display_set_supported_session_types (self, g_value_get_boxed (value));
+                break;
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
                 break;
@@ -1046,6 +1060,9 @@ gdm_display_get_property (GObject        *object,
         case PROP_ALLOW_TIMED_LOGIN:
                 g_value_set_boolean (value, priv->allow_timed_login);
                 break;
+        case PROP_SUPPORTED_SESSION_TYPES:
+                g_value_set_boxed (value, priv->supported_session_types);
+                break;
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
                 break;
@@ -1057,13 +1074,12 @@ handle_get_id (GdmDBusDisplay        *skeleton,
                GDBusMethodInvocation *invocation,
                GdmDisplay            *self)
 {
-        char *id;
+        g_autofree char *id = NULL;
 
         gdm_display_get_id (self, &id, NULL);
 
         gdm_dbus_display_complete_get_id (skeleton, invocation, id);
 
-        g_free (id);
         return TRUE;
 }
 
@@ -1072,7 +1088,7 @@ handle_get_remote_hostname (GdmDBusDisplay        *skeleton,
                             GDBusMethodInvocation *invocation,
                             GdmDisplay            *self)
 {
-        char *hostname;
+        g_autofree char *hostname = NULL;
 
         gdm_display_get_remote_hostname (self, &hostname, NULL);
 
@@ -1080,7 +1096,6 @@ handle_get_remote_hostname (GdmDBusDisplay        *skeleton,
                                                        invocation,
                                                        hostname ? hostname : "");
 
-        g_free (hostname);
         return TRUE;
 }
 
@@ -1089,9 +1104,8 @@ handle_get_seat_id (GdmDBusDisplay        *skeleton,
                     GDBusMethodInvocation *invocation,
                     GdmDisplay            *self)
 {
-        char *seat_id;
+        g_autofree char *seat_id = NULL;
 
-        seat_id = NULL;
         gdm_display_get_seat_id (self, &seat_id, NULL);
 
         if (seat_id == NULL) {
@@ -1099,7 +1113,6 @@ handle_get_seat_id (GdmDBusDisplay        *skeleton,
         }
         gdm_dbus_display_complete_get_seat_id (skeleton, invocation, seat_id);
 
-        g_free (seat_id);
         return TRUE;
 }
 
@@ -1108,13 +1121,12 @@ handle_get_x11_display_name (GdmDBusDisplay        *skeleton,
                              GDBusMethodInvocation *invocation,
                              GdmDisplay            *self)
 {
-        char *name;
+        g_autofree char *name = NULL;
 
         gdm_display_get_x11_display_name (self, &name, NULL);
 
         gdm_dbus_display_complete_get_x11_display_name (skeleton, invocation, name);
 
-        g_free (name);
         return TRUE;
 }
 
@@ -1150,15 +1162,13 @@ static gboolean
 register_display (GdmDisplay *self)
 {
         GdmDisplayPrivate *priv;
-        GError *error = NULL;
+        g_autoptr(GError) error = NULL;
 
         priv = gdm_display_get_instance_private (self);
 
-        error = NULL;
         priv->connection = g_bus_get_sync (G_BUS_TYPE_SYSTEM, NULL, &error);
         if (priv->connection == NULL) {
                 g_critical ("error getting system bus: %s", error->message);
-                g_error_free (error);
                 exit (EXIT_FAILURE);
         }
 
@@ -1226,11 +1236,9 @@ gdm_display_dispose (GObject *object)
 
         g_debug ("GdmDisplay: Disposing display");
 
-        if (priv->finish_idle_id != 0) {
-                g_source_remove (priv->finish_idle_id);
-                priv->finish_idle_id = 0;
-        }
+        g_clear_handle_id (&priv->finish_idle_id, g_source_remove);
         g_clear_object (&priv->launch_environment);
+        g_clear_pointer (&priv->supported_session_types, g_strfreev);
 
         g_warn_if_fail (priv->status != GDM_DISPLAY_MANAGED);
         g_warn_if_fail (priv->user_access_file == NULL);
@@ -1391,6 +1399,14 @@ gdm_display_class_init (GdmDisplayClass *klass)
                                                            G_MAXINT,
                                                            GDM_DISPLAY_UNMANAGED,
                                                            G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_STRINGS));
+
+        g_object_class_install_property (object_class,
+                                         PROP_SUPPORTED_SESSION_TYPES,
+                                         g_param_spec_boxed ("supported-session-types",
+                                                             "supported session types",
+                                                             "supported session types",
+                                                             G_TYPE_STRV,
+                                                             G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_STRINGS));
 }
 
 static void
@@ -1446,6 +1462,8 @@ gdm_display_get_object_skeleton (GdmDisplay *self)
 {
         GdmDisplayPrivate *priv;
 
+        g_return_val_if_fail (GDM_IS_DISPLAY (self), NULL);
+
         priv = gdm_display_get_instance_private (self);
         return priv->object_skeleton;
 }
@@ -1454,12 +1472,11 @@ static void
 on_launch_environment_session_opened (GdmLaunchEnvironment *launch_environment,
                                       GdmDisplay           *self)
 {
-        char       *session_id;
+        g_autofree char *session_id = NULL;
 
         g_debug ("GdmDisplay: Greeter session opened");
         session_id = gdm_launch_environment_get_session_id (launch_environment);
-        _gdm_display_set_session_id (self, session_id);
-        g_free (session_id);
+        g_object_set (G_OBJECT (self), "session-id", session_id, NULL);
 }
 
 static void
@@ -1473,9 +1490,9 @@ static void
 self_destruct (GdmDisplay *self)
 {
         g_object_ref (self);
-        if (gdm_display_get_status (self) == GDM_DISPLAY_MANAGED) {
-                gdm_display_unmanage (self);
-        }
+
+        g_debug ("GdmDisplay: initiating display self-destruct");
+        gdm_display_unmanage (self);
 
         if (gdm_display_get_status (self) != GDM_DISPLAY_FINISHED) {
                 queue_finish (self);
@@ -1512,14 +1529,12 @@ on_launch_environment_session_died (GdmLaunchEnvironment *launch_environment,
 static gboolean
 can_create_environment (const char *session_id)
 {
-        char *path;
+        g_autofree char *path = NULL;
         gboolean session_exists;
 
         path = g_strdup_printf (GNOME_SESSION_SESSIONS_PATH "/%s.session", session_id);
         session_exists = g_file_test (path, G_FILE_TEST_EXISTS);
 
-        g_free (path);
-
         return session_exists;
 }
 
@@ -1539,9 +1554,9 @@ kernel_cmdline_initial_setup_argument (const gchar  *contents,
                                        gchar       **initial_setup_argument,
                                        GError      **error)
 {
-        GRegex *regex = NULL;
-        GMatchInfo *match_info = NULL;
-        gchar *match_group = NULL;
+        g_autoptr(GRegex) regex = NULL;
+        g_autoptr(GMatchInfo) match_info = NULL;
+        g_autofree gchar *match_group = NULL;
 
         g_return_val_if_fail (initial_setup_argument != NULL, FALSE);
 
@@ -1551,9 +1566,6 @@ kernel_cmdline_initial_setup_argument (const gchar  *contents,
             return FALSE;
 
         if (!g_regex_match (regex, contents, 0, &match_info)) {
-                g_free (match_info);
-                g_free (regex);
-
                 g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
                              "Could not match gnome.initial-setup= in kernel cmdline");
 
@@ -1563,19 +1575,13 @@ kernel_cmdline_initial_setup_argument (const gchar  *contents,
         match_group = g_match_info_fetch (match_info, 1);
 
         if (!match_group) {
-                g_free (match_info);
-                g_free (regex);
-
                 g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
                              "Could not match gnome.initial-setup= in kernel cmdline");
 
                 return FALSE;
         }
 
-        *initial_setup_argument = match_group;
-
-        g_free (match_info);
-        g_free (regex);
+        *initial_setup_argument = g_steal_pointer (&match_group);
 
         return TRUE;
 }
@@ -1585,16 +1591,15 @@ kernel_cmdline_initial_setup_argument (const gchar  *contents,
 static gboolean
 kernel_cmdline_initial_setup_force_state (gboolean *force_state)
 {
-        GError *error = NULL;
-        gchar *contents = NULL;
-        gchar *setup_argument = NULL;
+        g_autoptr(GError) error = NULL;
+        g_autofree gchar *contents = NULL;
+        g_autofree gchar *setup_argument = NULL;
 
         g_return_val_if_fail (force_state != NULL, FALSE);
 
         if (!g_file_get_contents ("/proc/cmdline", &contents, NULL, &error)) {
                 g_debug ("GdmDisplay: Could not check kernel parameters, not forcing initial setup: %s",
                           error->message);
-                g_clear_error (&error);
                 return FALSE;
         }
 
@@ -1602,16 +1607,12 @@ kernel_cmdline_initial_setup_force_state (gboolean *force_state)
 
         if (!kernel_cmdline_initial_setup_argument (contents, &setup_argument, &error)) {
                 g_debug ("GdmDisplay: Failed to read kernel commandline: %s", error->message);
-                g_clear_pointer (&contents, g_free);
                 return FALSE;
         }
 
-        g_clear_pointer (&contents, g_free);
-
         /* Poor-man's check for truthy or falsey values */
         *force_state = setup_argument[0] == '1';
 
-        g_free (setup_argument);
         return TRUE;
 }
 
@@ -1634,7 +1635,7 @@ wants_initial_setup (GdmDisplay *self)
                         return TRUE;
                 }
 
-                g_debug ("GdmDisplay: Forceing no gnome-initial-setup");
+                g_debug ("GdmDisplay: Forcing no gnome-initial-setup");
                 return FALSE;
         }
 
@@ -1666,20 +1667,18 @@ gdm_display_start_greeter_session (GdmDisplay *self)
 {
         GdmDisplayPrivate *priv;
         GdmSession    *session;
-        char          *display_name;
-        char          *seat_id;
-        char          *hostname;
-        char          *auth_file = NULL;
+        g_autofree char *display_name = NULL;
+        g_autofree char *seat_id = NULL;
+        g_autofree char *hostname = NULL;
+        g_autofree char *auth_file = NULL;
+
+        g_return_if_fail (GDM_IS_DISPLAY (self));
 
         priv = gdm_display_get_instance_private (self);
         g_return_if_fail (g_strcmp0 (priv->session_class, "greeter") == 0);
 
         g_debug ("GdmDisplay: Running greeter");
 
-        display_name = NULL;
-        seat_id = NULL;
-        hostname = NULL;
-
         g_object_get (self,
                       "x11-display-name", &display_name,
                       "seat-id", &seat_id,
@@ -1723,12 +1722,8 @@ gdm_display_start_greeter_session (GdmDisplay *self)
         session = gdm_launch_environment_get_session (priv->launch_environment);
         g_object_set (G_OBJECT (session),
                       "display-is-initial", priv->is_initial,
+                      "supported-session-types", priv->supported_session_types,
                       NULL);
-
-        g_free (display_name);
-        g_free (seat_id);
-        g_free (hostname);
-        g_free (auth_file);
 }
 
 void
@@ -1736,6 +1731,8 @@ gdm_display_stop_greeter_session (GdmDisplay *self)
 {
         GdmDisplayPrivate *priv;
 
+        g_return_if_fail (GDM_IS_DISPLAY (self));
+
         priv = gdm_display_get_instance_private (self);
 
         if (priv->launch_environment != NULL) {
@@ -1793,10 +1790,8 @@ gdm_display_set_windowpath (GdmDisplay *self)
         xcb_get_property_reply_t *get_property_reply = NULL;
         xcb_window_t root_window = XCB_WINDOW_NONE;
         const char *windowpath;
-        char *newwindowpath;
+        g_autofree gchar *newwindowpath = NULL;
         uint32_t num;
-        char nums[10];
-        int numn;
 
         priv = gdm_display_get_instance_private (self);
 
@@ -1834,13 +1829,10 @@ gdm_display_set_windowpath (GdmDisplay *self)
         num = ((uint32_t *) xcb_get_property_value (get_property_reply))[0];
 
         windowpath = getenv ("WINDOWPATH");
-        numn = snprintf (nums, sizeof (nums), "%u", num);
         if (!windowpath) {
-                newwindowpath = malloc (numn + 1);
-                sprintf (newwindowpath, "%s", nums);
+                newwindowpath = g_strdup_printf ("%u", num);
         } else {
-                newwindowpath = malloc (strlen (windowpath) + 1 + numn + 1);
-                sprintf (newwindowpath, "%s:%s", windowpath, nums);
+                newwindowpath = g_strdup_printf ("%s:%u", windowpath, num);
         }
 
         g_setenv ("WINDOWPATH", newwindowpath, TRUE);
@@ -1856,8 +1848,9 @@ gdm_display_connect (GdmDisplay *self)
         xcb_auth_info_t *auth_info = NULL;
         gboolean ret;
 
+        g_return_val_if_fail (GDM_IS_DISPLAY (self), FALSE);
+
         priv = gdm_display_get_instance_private (self);
-        ret = FALSE;
 
         g_debug ("GdmDisplay: Server is ready - opening display %s", priv->x11_display_name);
 
